@@ -1,20 +1,39 @@
-sanity_newdata <- function(model, newdata) {
+sanitize_newdata <- function(model, newdata) {
 
     checkmate::assert(
         checkmate::check_data_frame(newdata, null.ok = TRUE),
-        checkmate::check_choice(newdata, choices = c("mean", "median", "marginalmeans")),
+        checkmate::check_choice(newdata, choices = c("mean", "median", "tukey", "grid", "marginalmeans")),
         combine = "or")
 
-    if (is.null(newdata)) {
-        newdata <- suppressWarnings(insight::get_data(model))
+    # we always need this to extract attributes
+    modeldata <- hush(insight::get_data(model))
 
-    } else if (isTRUE(checkmate::check_choice(newdata, "mean"))) {
+    if (is.null(newdata)) {
+        newdata <- modeldata
+
+    } else if (identical(newdata, "mean")) {
         newdata <- datagrid(model = model)
 
-    } else if (isTRUE(checkmate::check_choice(newdata, "median"))) {
-        newdata <- datagrid(model = model, FUN.numeric = function(x) stats::median(x, na.rm = TRUE))
+    } else if (identical(newdata, "median")) {
+        newdata <- datagrid(
+            model = model,
+            FUN.numeric = function(x) stats::median(x, na.rm = TRUE))
 
-    } else if (isTRUE(checkmate::check_choice(newdata, "marginalmeans"))) {
+    } else if (identical(newdata, "tukey")) {
+        newdata <- datagrid(
+            model = model,
+            FUN.numeric = function(x) stats::fivenum(x, na.rm = TRUE))
+
+    } else if (identical(newdata, "grid")) {
+        newdata <- datagrid(
+            model = model,
+            FUN.numeric = function(x) stats::fivenum(x, na.rm = TRUE),
+            FUN.factor = unique,
+            FUN.character = unique,
+            FUN.logical = unique)
+
+    # grid with all unique values of categorical variables, and numerics at their means
+    } else if (identical(newdata, "marginalmeans")) {
         newdata <- datagrid(
             model = model,
             FUN.factor = unique,
@@ -23,12 +42,28 @@ sanity_newdata <- function(model, newdata) {
     }
 
     if (!inherits(newdata, "data.frame")) {
-        msg <- sprintf("Unable to extract the data from model of class `%s`. This can happen in a variety of cases, such as when a `marginaleffects` package function is called from inside a user-defined function. Please supply a data frame explicitly via the `newdata` argument.", class(model)[1])
+        msg <- format_msg(
+        "Unable to extract the data from model of class `%s`. This can happen in a
+        variety of cases, such as when a `marginaleffects` package function is called
+        from inside a user-defined function. Please supply a data frame explicitly via
+        the `newdata` argument.")
+        msg <- sprintf(msg, class(model)[1])
         stop(msg, call. = FALSE)
     }
 
-    # matrix columns are only partially supported
-    matrix_columns <- names(newdata)[sapply(newdata, is.matrix)]
+    # column attributes
+    mc <- Filter(function(x) is.matrix(modeldata[[x]]), colnames(modeldata))
+    cl <- Filter(function(x) is.character(modeldata[[x]]), colnames(modeldata))
+    cl <- lapply(modeldata[, cl], unique)
+    vc <- sapply(names(modeldata), find_variable_class, newdata = modeldata, model = model)
+    column_attributes <- list(
+        "matrix_columns" = mc,
+        "character_levels" = cl,
+        "variable_class" = vc)
+
+    # {modelbased} sometimes attaches useful attributes
+    exclude <- c("class", "row.names", "names", "data", "reference")
+    modelbased_attributes <- get_attributes(newdata, exclude = exclude)
 
     # required for the type of column indexing to follow
     data.table::setDF(newdata)
@@ -44,8 +79,8 @@ sanity_newdata <- function(model, newdata) {
     # rbindlist breaks on matrix columns
     idx <- sapply(newdata, function(x) class(x)[1] == "matrix")
     if (any(idx)) {
-        # unpacking matrix columns works with {mgcv} but breaks {mclogit}
         # Issue #363
+        # unpacking matrix columns works with {mgcv} but breaks {mclogit}
         if (inherits(model, "gam")) {
             newdata <- unpack_matrix_cols(newdata)
         } else {
@@ -64,14 +99,11 @@ sanity_newdata <- function(model, newdata) {
             flag <- TRUE
         }
     }
-    # This may be useful but it raises way too many warnings
-    #} else {
-    #     for (cv in categorical_variables) {
-    #         if (is.numeric(newdata[[cv]])) {
-    #             flag <- TRUE
-    #         }
-    #     }
-    # }
+
+    # attributes
+    newdata <- set_attributes(newdata, modelbased_attributes, prefix = "newdata_")
+    newdata <- set_attributes(newdata, column_attributes, prefix = "newdata_")
+    attr(newdata, "newdata_modeldata") <- modeldata
 
     # we will need this to merge the original data back in, and it is better to
     # do it in a centralized upfront way.
@@ -79,9 +111,8 @@ sanity_newdata <- function(model, newdata) {
         newdata$rowid <- seq_len(nrow(newdata))
     }
 
-    attr(newdata, "matrix_columns") <- matrix_columns
-
     return(newdata)
 }
 
 
+
