@@ -28,10 +28,13 @@ sanity_model_supported_class <- function(model) {
         "afex_aov",
         "betareg",
         "bife",
+        "biglm",
+        c("bigglm", "biglm"),
         "brglmFit",
         "brmsfit",
         c("bracl", "brmultinom", "brglmFit"),
         c("brnb", "negbin", "glm"),
+        c("clogit", "coxph"),
         "clm",
         "coxph",
         "crch",
